//=======================================================================================
// ____          ____    __    ______     __________   __      __       __        __
// \    \       |    |  |  |  |   _   \  |___    ___| |  |    |  |     /  \      |  |
//  \    \      |    |  |  |  |  |_)   |     |  |     |  |    |  |    /    \     |  |
//   \    \     |    |  |  |  |   _   /      |  |     |  |    |  |   /  /\  \    |  |
//    \    \    |    |  |  |  |  | \  \      |  |     |   \__/   |  /  ____  \   |  |____
//     \    \   |    |  |__|  |__|  \__\     |__|      \________/  /__/    \__\  |_______|
//      \    \  |    |   ________________________________________________________________
//       \    \ |    |  |  ______________________________________________________________|
//        \    \|    |  |  |         __          __     __     __     ______      _______
//         \         |  |  |_____   |  |        |  |   |  |   |  |   |   _  \    /  _____)
//          \        |  |   _____|  |  |        |  |   |  |   |  |   |  | \  \   \_______
//           \       |  |  |        |  |_____   |   \_/   |   |  |   |  |_/  /    _____  |
//            \ _____|  |__|        |________|   \_______/    |__|   |______/    (_______/
//
//  This file is part of VirtualFluids. VirtualFluids is free software: you can
//  redistribute it and/or modify it under the terms of the GNU General Public
//  License as published by the Free Software Foundation, either version 3 of
//  the License, or (at your option) any later version.
//
//  VirtualFluids is distributed in the hope that it will be useful, but WITHOUT
//  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
//  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
//  for more details.
//
//  SPDX-License-Identifier: GPL-3.0-or-later
//  SPDX-FileCopyrightText: Copyright © VirtualFluids Project contributors, see AUTHORS.md in root folder
//
//! \addtogroup NumericalTests
//! \ingroup numerical_tests
//! \{
//=======================================================================================
#ifndef SIMULATION_PARAMETER_H
#define SIMULATION_PARAMETER_H

#include <memory>
#include <string>
#include <vector>

class InitialCondition;
class KernelConfiguration;

class SimulationParameter
{
public:
    virtual ~SimulationParameter() = default;
    virtual std::shared_ptr<KernelConfiguration> getKernelConfiguration() = 0;
    virtual double getViscosity() = 0;
    virtual std::string getGridPath() = 0;
    virtual std::string getFilePath() = 0;
    virtual unsigned int getNumberOfGridLevels() = 0;
    virtual unsigned int getEndTime() = 0;
    virtual unsigned int getTimeStepLength() = 0;
    virtual std::vector<unsigned int> getDevices() = 0;
    virtual double getMaxVelocity() = 0;

    virtual unsigned int getLx() = 0;
    virtual unsigned int getLz() = 0;
    virtual unsigned int getL0() = 0; 
};

#endif

//! \}
